/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.sidebox;

import java.sql.*;
import java.util.*;
import org.apache.log4j.Logger;
import com.silverwrist.util.*;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.util.*;

public class SideboxOps_mysql extends SideboxOps
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static final Logger logger = Logger.getLogger(SideboxOps_mysql.class);

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private PropertySerializer m_psz;        // reference to property serializer

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  public SideboxOps_mysql(DBConnectionPool pool)
  {
    super(pool);
    m_psz = (PropertySerializer)(pool.queryService(PropertySerializer.class));

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  private final String serializeContextParam(Object param) throws DatabaseException
  {
    if (param==null)
      return null;
    String s = m_psz.serializeProperty(param);
    if (s==null)
    { // serialization exception - throw it
      DatabaseException de = new DatabaseException(SideboxOps_mysql.class,"SideboxMessages","context.param.ser");
      de.setParameter(0,param.getClass().getName());
      throw de;

    } // end if

    return s;

  } // end serializeContextParam

  /*--------------------------------------------------------------------------------
   * Overrides from class OpsBase
   *--------------------------------------------------------------------------------
   */

  public void dispose()
  {
    m_psz = null;
    super.dispose();

  } // end dispose

  /*--------------------------------------------------------------------------------
   * Abstract implementations from class SideboxOps
   *--------------------------------------------------------------------------------
   */

  int[] getSideboxIDList(int uid, int context_nsid, String context_name, Object context_param) throws DatabaseException
  {
    String context_param_ser = serializeContextParam(context_param);
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    LinkedList tmp_list = new LinkedList();
    int total = 0;

    if (logger.isDebugEnabled())
      logger.debug("Getting sidebox list for UID #" + uid);

    try
    { // get a connection
      conn = getConnection();

      // create and execute the statement
      if (context_param_ser==null)
	stmt = conn.prepareStatement("SELECT sbid FROM sbox_deploy WHERE uid = ? AND ctx_nsid = ? AND ctx_name = ? "
				     + "AND param IS NULL ORDER BY seq;");
      else
      { // use the version with the parameter
	stmt = conn.prepareStatement("SELECT sbid FROM sbox_deploy WHERE uid = ? AND ctx_nsid = ? AND ctx_name = ? "
				     + "AND param = ? ORDER BY seq;");
	stmt.setString(4,context_param_ser);

      } // end else

      stmt.setInt(1,uid);
      stmt.setInt(2,context_nsid);
      stmt.setString(3,context_name);
      rs = stmt.executeQuery();

      // build the temporary return arrays
      int curptr = 0;
      int[] cur = new int[16];
      while (rs.next())
      { // add elements to the array
	if (curptr==16)
	{ // shift array to list
	  tmp_list.addLast(cur);
	  cur = new int[16];
	  curptr = 0;

	} // end if

	cur[curptr++] = rs.getInt(1);
	total++;

      } // end while

      tmp_list.addLast(cur);  // add the "tail" to the array

      if (logger.isDebugEnabled())
	logger.debug("found: " + total + " sidebox IDs");

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

    // Build the return array.
    int[] rc = new int[total];
    int outptr = 0;
    while (total>0)
    { // copy array segments into the full return array
      int[] piece = (int[])(tmp_list.removeFirst());
      int ncopy = Math.min(total,16);
      System.arraycopy(piece,0,rc,outptr,ncopy);
      outptr += ncopy;
      total -= ncopy;

    } // end while

    tmp_list.clear();
    return rc;

  } // end getSideboxIDList

  PropertyKey getSideboxType(int sbid) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // create and execute the statement
      stmt = conn.prepareStatement("SELECT type_nsid, type_name FROM sbox_master WHERE sbid = ?;");
      stmt.setInt(1,sbid);
      rs = stmt.executeQuery();
      if (rs.next())
	return new PropertyKey(rs.getInt(1),rs.getString(2));

      // throw an exception if we didn't find it
      DatabaseException de = new DatabaseException(SideboxOps_mysql.class,"SideboxMessages","no.sidebox");
      de.setParameter(0,String.valueOf(sbid));
      throw de;

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end getSideboxType

  PropertyKey getSideboxIdentity(int sbid) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // create and execute the statement
      stmt = conn.prepareStatement("SELECT sb_nsid, sb_name FROM sbox_master WHERE sbid = ?;");
      stmt.setInt(1,sbid);
      rs = stmt.executeQuery();
      if (rs.next())
	return new PropertyKey(rs.getInt(1),rs.getString(2));

      // throw an exception if we didn't find it
      DatabaseException de = new DatabaseException(SideboxOps_mysql.class,"SideboxMessages","no.sidebox");
      de.setParameter(0,String.valueOf(sbid));
      throw de;

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end getSideboxIdentity

  Object getSideboxProperty(int sbid, PropertyKey key) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    String rc_str = null;
    try
    { // get a connection
      conn = getConnection();

      // look up the property
      stmt = conn.prepareStatement("SELECT prop_value FROM sbox_props WHERE sbid = ? AND nsid = ? AND prop_name = ?;");
      stmt.setInt(1,sbid);
      stmt.setInt(2,key.getNamespaceID());
      stmt.setString(3,key.getName());
      rs = stmt.executeQuery();
      if (!(rs.next()))
	return null;  // property not found

      rc_str = rs.getString(1);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

    // Deserialize the property value.
    Object rc = m_psz.deserializeProperty(rc_str);
    if (rc!=null)
      return rc;

    // deserialization exception - throw it
    DatabaseException de = new DatabaseException(SideboxOps_mysql.class,"SideboxMessages","property.deserialize");
    de.setParameter(0,key.getName());
    throw de;

  } // end getSideboxProperty

} // end class SideboxOps_mysql
